# This file was auto-generated by Fern from our API Definition.

import typing

from ...attachments.types.attachment_id import AttachmentId
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...threads.types.list_threads_response import ListThreadsResponse
from ...threads.types.thread import Thread
from ...threads.types.thread_id import ThreadId
from ...types.ascending import Ascending
from ...types.labels import Labels
from ...types.limit import Limit
from ...types.page_token import PageToken
from ..types.inbox_id import InboxId
from .raw_client import AsyncRawThreadsClient, RawThreadsClient


class ThreadsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawThreadsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawThreadsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawThreadsClient
        """
        return self._raw_client

    def list(
        self,
        inbox_id: InboxId,
        *,
        limit: typing.Optional[Limit] = None,
        page_token: typing.Optional[PageToken] = None,
        labels: typing.Optional[Labels] = None,
        ascending: typing.Optional[Ascending] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListThreadsResponse:
        """
        Parameters
        ----------
        inbox_id : InboxId

        limit : typing.Optional[Limit]

        page_token : typing.Optional[PageToken]

        labels : typing.Optional[Labels]

        ascending : typing.Optional[Ascending]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListThreadsResponse

        Examples
        --------
        from agentmail import AgentMail
        client = AgentMail(api_key="YOUR_API_KEY", )
        client.inboxes.threads.list(inbox_id='inbox_id', )
        """
        _response = self._raw_client.list(
            inbox_id,
            limit=limit,
            page_token=page_token,
            labels=labels,
            ascending=ascending,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, inbox_id: InboxId, thread_id: ThreadId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Thread:
        """
        Parameters
        ----------
        inbox_id : InboxId

        thread_id : ThreadId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Thread

        Examples
        --------
        from agentmail import AgentMail
        client = AgentMail(api_key="YOUR_API_KEY", )
        client.inboxes.threads.get(inbox_id='inbox_id', thread_id='thread_id', )
        """
        _response = self._raw_client.get(inbox_id, thread_id, request_options=request_options)
        return _response.data

    def get_attachment(
        self,
        inbox_id: InboxId,
        thread_id: ThreadId,
        attachment_id: AttachmentId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Parameters
        ----------
        inbox_id : InboxId

        thread_id : ThreadId

        attachment_id : AttachmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
        """
        with self._raw_client.get_attachment(inbox_id, thread_id, attachment_id, request_options=request_options) as r:
            yield from r.data


class AsyncThreadsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawThreadsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawThreadsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawThreadsClient
        """
        return self._raw_client

    async def list(
        self,
        inbox_id: InboxId,
        *,
        limit: typing.Optional[Limit] = None,
        page_token: typing.Optional[PageToken] = None,
        labels: typing.Optional[Labels] = None,
        ascending: typing.Optional[Ascending] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListThreadsResponse:
        """
        Parameters
        ----------
        inbox_id : InboxId

        limit : typing.Optional[Limit]

        page_token : typing.Optional[PageToken]

        labels : typing.Optional[Labels]

        ascending : typing.Optional[Ascending]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListThreadsResponse

        Examples
        --------
        from agentmail import AsyncAgentMail
        import asyncio
        client = AsyncAgentMail(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.inboxes.threads.list(inbox_id='inbox_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            inbox_id,
            limit=limit,
            page_token=page_token,
            labels=labels,
            ascending=ascending,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, inbox_id: InboxId, thread_id: ThreadId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Thread:
        """
        Parameters
        ----------
        inbox_id : InboxId

        thread_id : ThreadId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Thread

        Examples
        --------
        from agentmail import AsyncAgentMail
        import asyncio
        client = AsyncAgentMail(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.inboxes.threads.get(inbox_id='inbox_id', thread_id='thread_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get(inbox_id, thread_id, request_options=request_options)
        return _response.data

    async def get_attachment(
        self,
        inbox_id: InboxId,
        thread_id: ThreadId,
        attachment_id: AttachmentId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Parameters
        ----------
        inbox_id : InboxId

        thread_id : ThreadId

        attachment_id : AttachmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
        """
        async with self._raw_client.get_attachment(
            inbox_id, thread_id, attachment_id, request_options=request_options
        ) as r:
            async for data in r.data:
                yield data
